﻿#ifndef FDC_H_INCLUDED
#define FDC_H_INCLUDED

#include <windows.h>
#include "fdrawcmd.h"
#include "tdef.h"


#define	MAX_SECTOR					64		// 1トラックあたりの最大セクタ数

//************* Result Status 0 *************
#define ST0_NOT_READY				(0x08)
#define ST0_EQUIP_CHK				(0x10)
#define ST0_SEEK_END				(0x20)
#define ST0_IC_NT					(0x00)
#define ST0_IC_AT					(0x40)
#define ST0_IC_IC					(0x80)
#define ST0_IC_AI					(0xc0)

//************* Result Status 1 *************
#define ST1_MISSING_AM				(0x01)
#define ST1_NOT_WRITABLE			(0x02)
#define ST1_NO_DATA					(0x04)
#define ST1_DATA_ERROR				(0x20)

//************* Result Status 2 *************
#define ST2_MA_IN_DATA_FIELD		(0x01)
#define ST2_BAD_CYLINDER			(0x02)
#define ST2_NO_CYLINDER				(0x10)
#define ST2_DE_IN_DATA_FIELD		(0x20)
#define ST2_CONTROL_MARK			(0x40)



struct SECINF {
	BYTE c;							// ID の C (シリンダNo 片面の場合は=トラックNo)
	BYTE h;							// ID の H (ヘッダアドレス 片面の場合は=0)
	BYTE r;							// ID の R (トラック内のセクタNo)
	BYTE n;							// ID の N (セクタサイズ 0:128 1:256 2:512 3:1024)
	int secoff;						// セクタオフセット(同一トラック内に同IDのセクタがある場合の通し番号)
									// 1個しかなければ0, 2個以上ある場合は1,2,3...
	DWORD reltime;					// インデックスホールからの時間(us)
	BYTE density;					// 記録密度     0x00:倍密度   0x40:単密度
	
	SECINF() :
		c( 0 ), h( 0 ), r( 0 ), n( 0 ), secoff( 0 ), reltime( 0 ), density( 0 ) {}
};



////////////////////////////////////////////////////////////////
// クラス定義
////////////////////////////////////////////////////////////////
class FDC {
private:
	HANDLE fh;						// FDDのファイルハンドル
	
	FD_CMD_RESULT CmdRes;			// リザルトステータス
	FD_INTERRUPT_STATUS Ist;
	
	int Retry;						// リトライ回数
	
	
	DWORD GetDriverVersion();
	bool OpenFDD( int, BYTE );
	bool SendRawCommand( DWORD, LPVOID, DWORD, LPVOID, DWORD );
	bool SetDataRate( BYTE );
	bool Reset();
	bool GetResult();
	
public:
	FDC();							// コンストラクタ
	~FDC();							// デストラクタ
	
	int Init( int, BYTE );
	
	// Raw command
	bool CmdRecalibrate();
	bool CmdSeek( BYTE);
	bool CmdReadID( BYTE, BYTE = FD_OPTION_MFM );
	bool CmdReadData( BYTE, BYTE, BYTE, BYTE, BYTE, BYTE*, BYTE = FD_OPTION_MFM );
	bool CmdReadDeletedData( BYTE, BYTE, BYTE, BYTE, BYTE, BYTE*, BYTE = FD_OPTION_MFM );
	bool CmdWriteData( BYTE, BYTE, BYTE, BYTE, BYTE, BYTE*, BYTE = FD_OPTION_MFM );
	bool CmdWriteDeletedData( BYTE, BYTE, BYTE, BYTE, BYTE, BYTE*, BYTE = FD_OPTION_MFM );
	bool CmdFormat( PFD_FORMAT_PARAMS );
	bool CmdSpecify( BYTE, BYTE, BYTE );
	
	int ScanTrack( BYTE, SECINF*, BYTE = FD_OPTION_MFM );
	bool SetSectorOffset( BYTE = 0 );
	bool SetShortWrite( DWORD, int = 0 );
	bool CheckDrive();
	
	BYTE ST0();						// リザルトステータス取得
	BYTE ST1();
	BYTE ST2();
	
	void SetRetry( int );			// リトライ回数設定

};


#endif
